-- --------------------------------------------------------------------
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- --------------------------------------------------------------------
-- Copyright (c) 2001 by Lattice Semiconductor Corporation
-- --------------------------------------------------------------------
--
-- Permission:
--
--   Lattice Semiconductor grants permission to use this code for use
--   in synthesis for any Lattice programmable logic product.  Other
--   use of this code, including the selling or duplication of any
--   portion is strictly prohibited.
--
-- Disclaimer:
--
--   This VHDL or Verilog source code is intended as a design reference
--   which illustrates how these types of functions can be implemented.
--   It is the user's responsibility to verify their design for
--   consistency and functionality through the use of formal
--   verification methods.  Lattice Semiconductor provides no warranty
--   regarding the use or functionality of this code.
--
-- --------------------------------------------------------------------
--           
--                     Lattice Semiconductor Corporation
--                     5555 NE Moore Court
--                     Hillsboro, OR 97214
--                     U.S.A
--
--                     TEL: 1-800-Lattice (USA and Canada)
--                          408-826-6000 (other locations)
--
--                     web: http://www.latticesemi.com/
--                     email: techsupport@latticesemi.com
--
-- --------------------------------------------------------------------
--
--  Project:           Universal Asynchronous Receiver Transmitter
--  File:              uart_tx_tb.vhd
--  Title:             uart_tx_tb
--  Design Library:    IEEE, generics
--  Dependencies:      IEEE.std_logic_1164.all
--                     IEEE.numeric_std.all
--                     generics.components.all
--  Description:       VHDL test bench for UART_top transmitter testing
--                     There are 4 tests in different combinations:
--                     Test 1 : 5-bit data, even parity, 1 stop
--                     Test 2 : 5-bit data, even parity, 1.5 stop
--                     Test 3 : 5-bit data, odd parity, 1 stop
--                     Test 4 : 5-bit data, odd parity, 1.5 stop
--                     Test 5 : 5-bit data, stick even parity, 1 stop
--                     Test 6 : 5-bit data, stick even parity, 1.5 stop
--                     Test 7 : 5-bit data, stick odd parity, 1 stop
--                     Test 8 : 5-bit data, stick odd parity, 1.5 stop
--                     Test 9 : 5-bit data, no parity, 1 stop
--                     Test 10 : 5-bit data, no parity, 1.5 stop
--                     Test 11 : 6-bit data, even parity, 1 stop
--                     Test 12 : 6-bit data, even parity, 2 stop
--                     Test 13 : 6-bit data, odd parity, 1 stop
--                     Test 14 : 6-bit data, odd parity, 2 stop
--                     Test 15 : 6-bit data, stick even parity, 1 stop
--                     Test 16 : 6-bit data, stick even parity, 2 stop
--                     Test 17 : 6-bit data, stick odd parity, 1 stop
--                     Test 18 : 6-bit data, stick odd parity, 2 stop
--                     Test 19 : 6-bit data, no parity, 1 stop
--                     Test 20 : 6-bit data, no parity, 2 stop
--                     Test 21 : 7-bit data, even parity, 1 stop
--                     Test 22 : 7-bit data, even parity, 2 stop
--                     Test 23 : 7-bit data, odd parity, 1 stop
--                     Test 24 : 7-bit data, odd parity, 2 stop
--                     Test 25 : 7-bit data, stick even parity, 1 stop
--                     Test 26 : 7-bit data, stick even parity, 2 stop
--                     Test 27 : 7-bit data, stick odd parity, 1 stop
--                     Test 28 : 7-bit data, stick odd parity, 2 stop
--                     Test 29 : 7-bit data, no parity, 1 stop
--                     Test 30 : 7-bit data, no parity, 2 stop
--                     Test 31 : 8-bit data, even parity, 1 stop
--                     Test 32 : 8-bit data, even parity, 2 stop
--                     Test 33 : 8-bit data, odd parity, 1 stop
--                     Test 34 : 8-bit data, odd parity, 2 stop
--                     Test 35 : 8-bit data, stick even parity, 1 stop
--                     Test 36 : 8-bit data, stick even parity, 2 stop
--                     Test 37 : 8-bit data, stick odd parity, 1 stop
--                     Test 38 : 8-bit data, stick odd parity, 2 stop
--                     Test 39 : 8-bit data, no parity, 1 stop
--                     Test 40 : 8-bit data, no parity, 2 stop
--
-- --------------------------------------------------------------------
--
-- Revision History :
-- --------------------------------------------------------------------
--   Ver  :| Author            :| Mod. Date :| Changes Made:
--   V1.1 :| J.H.              :| 06/19/01  :| Support ispMACH 5000VG
--   V1.0 :| J.H.              :| 06/01/01  :| First Release
-- --------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity uart_tx_tb is
end uart_tx_tb;

architecture behavior of uart_tx_tb is 

  component Uart_top
    port(
      MR     : in  std_logic;
      MCLK   : in  std_logic;

      CS     : in  std_logic;
      RDn    : in  std_logic;
      WRn    : in  std_logic;
      ADSn   : in  std_logic;
      A      : in  std_logic_vector(2 downto 0);

      DIN    : in  std_logic_vector(7 downto 0);
      DOUT   : out std_logic_vector(7 downto 0);
      DDIS   : out std_logic;
      INTR   : out std_logic;

      SIN    : in  std_logic;
      RxRDYn : out std_logic;

      SOUT   : out std_logic;      
      TxRDYn : out std_logic;

      DCDn   : in  std_logic;
      CTSn   : in  std_logic;
      DSRn   : in  std_logic;
      RIn    : in  std_logic;    
      DTRn   : out std_logic;
      RTSn   : out std_logic
    );
  end component;

  -- Clock Frequency of Test
  --   MCLK_MHZ : frequency of MCLK
  --   PLLO_MHZ : frequency of PLL clk_out
  --              When the UART design is targeted to devices without
  --              PLL feature or the PLL feature is not used, PLLO_MHZ
  --              should be set to the same value of MCLK_MHZ.
  constant MCLK_MHZ : real := 28.4000;
  constant PLLO_MHZ : real := 71.0000;

  constant ONE_K_NS : time := 1000 ns;

  -- Clock Period Declaration
  --   MCLK_CLK_PERIOD : clock period of MCLK
  --   CLK_PEROID      : clock period of internal clock
  constant MCLK_CLK_PERIOD : time := ONE_K_NS / MCLK_MHZ;
  constant CLK_PERIOD      : time := ONE_K_NS / PLLO_MHZ;

  -- UART Registers Address Map
  constant RBR : std_logic_vector(2 downto 0) := "000";
  constant THR : std_logic_vector(2 downto 0) := "000";
  constant IER : std_logic_vector(2 downto 0) := "001";
  constant IIR : std_logic_vector(2 downto 0) := "010";
  constant LCR : std_logic_vector(2 downto 0) := "011";
  constant MCR : std_logic_vector(2 downto 0) := "100";
  constant LSR : std_logic_vector(2 downto 0) := "101";
  constant MSR : std_logic_vector(2 downto 0) := "110";

  -- TimeOut Definition
  constant WAIT_TIMEOUT : integer := 1000;

  -- This procedure performs a write cycle over the internal register
  procedure write_reg (
    addr        : in  std_logic_vector(2 downto 0);
    data        : in  std_logic_vector(7 downto 0);
    signal CS   : out std_logic;
    signal ADSn : out std_logic;
    signal WRn  : out std_logic;
    signal A    : out std_logic_vector(2 Downto 0);
    signal DIN  : out std_logic_vector(7 DownTo 0)) is
  begin
    wait for CLK_PERIOD;
    ADSn <= '0';
    wait for CLK_PERIOD;
    A <= addr;
    CS <= '1';
    wait for CLK_PERIOD;
    ADSn <= '1';
    wait for CLK_PERIOD;
    A <= (others => '1');
    CS <= '0';
    wait for (2*CLK_PERIOD);
    WRn <= '0';
    wait for CLK_PERIOD;
    DIN <= data;
    wait for CLK_PERIOD;
    WRn <= '1';
    wait for CLK_PERIOD;
    DIN <= (others => '1');
    wait for (2*CLK_PERIOD);
  end write_reg;

  -- This procedure performs a read cycle over the internal register
  procedure read_reg (
    addr        : in  std_logic_vector(2 downto 0);
    signal data : out std_logic_vector(7 downto 0);
    signal CS   : out std_logic;
    signal ADSn : out std_logic;
    signal RDn  : out std_logic;
    signal A    : out std_logic_vector(2 Downto 0);
    signal DOUT : in  std_logic_vector(7 Downto 0)) is
  begin
    wait for CLK_PERIOD;
    ADSn <= '0';
    wait for CLK_PERIOD;
    A <= addr;
    CS <= '1';
    wait for CLK_PERIOD;
    ADSn <= '1';
    wait for CLK_PERIOD;
    A <= (others => '1');
    CS <= '0';
    wait for (2*CLK_PERIOD);
    RDn <= '0';
    wait for (2*CLK_PERIOD);
    data <= DOUT;
    RDn <= '1';
    wait for (3*CLK_PERIOD);
  end read_reg;

  -- This procedure generates a serial frame for testing
  procedure sout_chk (
    numDataBits        : integer range 5 to 8;
    Txdata             : in std_logic_vector(7 downto 0);
    ParityBit          : in std_logic;
    stopBitLength      : real;
    parityBitExist     : boolean;
    constant cycleTime : in time;
    signal SOUT        : in std_logic) is
    variable i : integer;
  begin
    -- Wait for Start Bit
    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for cycleTime/32;
        exit when SOUT = '0';
        i := i + 1;
      else
        assert (false) report"Start bit Generation Failed"
        severity failure;
      end if;
    end loop;
    -- Start Bit checking
    for i in 1 to 15 loop
      wait for cycleTime/16;
      assert SOUT = '0'
        report "Start bit too short"
        severity failure;
    end loop;
    -- Data Bits checking
    for dataBit in 0 to numDataBits-1 loop
      for i in 0 to 15 loop
        wait for cycleTime/16;
        assert SOUT = TxData(databit)
          report "Transmitted Data bits incorrect"
          severity failure;
      end loop;
    end loop;
    -- Parity Bit checking
    if (parityBitExist) then
      for i in 0 to 15 loop
        wait for cycleTime/16;
        assert SOUT = ParityBit
          report "Transmitted Parity bit incorrect"
          severity failure;
      end loop;
    end if;
    -- Stop Bit checking
    if (stopBitLength = 1.0) then
      for i in 0 to 15 loop
        wait for cycleTime/16;
        assert SOUT = '1'
          report "Transmitted 1 Stop bit incorrect"
          severity failure;
      end loop;
    elsif (stopBitLength = 1.5) then
      for i in 0 to 23 loop
        wait for cycleTime/16;
        assert SOUT = '1'
          report "Transmitted 1.5 Stop bit incorrect"
          severity failure;
      end loop;
    elsif (stopBitLength = 2.0) then
      for i in 0 to 31 loop
        wait for cycleTime/16;
        assert SOUT = '1'
          report "Transmitted 2 Stop bit incorrect"
          severity failure;
      end loop;
    else
      assert (false)
        report "Incorrect Stop bit length specified"
        severity failure;
    end if;
  end sout_chk;

  signal MR     : std_logic := '1';
  signal MCLK   : std_logic := '0';
  signal CS     : std_logic := '0';
  signal RDn    : std_logic := '1';
  signal WRn    : std_logic := '1';
  signal ADSn   : std_logic := '1';
  signal A      : std_logic_vector(2 downto 0);
  signal DIN    : std_logic_vector(7 downto 0);
  signal DOUT   : std_logic_vector(7 downto 0);
  signal DDIS   : std_logic;
  signal INTR   : std_logic;
  signal SIN    : std_logic;
  signal RxRDYn : std_logic;
  signal SOUT   : std_logic;
  signal TxRDYn : std_logic;
  signal DCDn   : std_logic;
  signal CTSn   : std_logic;
  signal DSRn   : std_logic;
  signal RIn    : std_logic;
  signal DTRn   : std_logic;
  signal RTSn   : std_logic;

  signal regData_readBack : std_logic_vector(7 downto 0);
  signal TestID           : integer := 0;

  signal PCLK   : std_logic := '0';

begin


-----------------------------------------------------------------------
-- UUT Instantiation
-----------------------------------------------------------------------
  uut: Uart_top port map(
    MR     => MR,
    MCLK   => MCLK,
    CS     => CS,
    RDn    => RDn,
    WRn    => WRn,
    ADSn   => ADSn,
    A      => A,
    DIN    => DIN,
    DOUT   => DOUT,
    DDIS   => DDIS,
    INTR   => INTR,
    SIN    => SIN,
    RxRDYn => RxRDYn,
    SOUT   => SOUT,
    TxRDYn => TxRDYn,
    DCDn   => DCDn,
    CTSn   => CTSn,
    DSRn   => DSRn,
    RIn    => RIn,
    DTRn   => DTRn,
    RTSn   => RTSn
  );


-- Master Clock Generator
  MCLK <= not MCLK after (MCLK_CLK_PERIOD/2);

-- PLL Clock Generator (for simulation purpose)
  PCLK <= not PCLK after (CLK_PERIOD/2);


-----------------------------------------------------------------------
-- SOUT Checking for UART Transmitter Functions Tests
-----------------------------------------------------------------------
  Sout_Chk_Proc: process
  begin

    wait for (10*CLK_PERIOD);
    
    -- Test 1 ----------------------------------------------------
    --   5-bit data, even parity, 1 stop
    sout_chk(5, "01010101", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10101010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "01011010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10100101", '0', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 2 ----------------------------------------------------
    --   5-bit data, even parity, 1.5 stop
    sout_chk(5, "01010101", '1', 1.5, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10101010", '0', 1.5, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "01011010", '1', 1.5, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10100101", '0', 1.5, True, CLK_PERIOD*16, SOUT);

    -- Test 3 ----------------------------------------------------
    --   5-bit data, odd parity, 1 stop
    sout_chk(5, "01010101", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10101010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "01011010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10100101", '1', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 4 ----------------------------------------------------
    --   5-bit data, odd parity, 1.5 stop
    sout_chk(5, "01010101", '0', 1.5, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10101010", '1', 1.5, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "01011010", '0', 1.5, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10100101", '1', 1.5, True, CLK_PERIOD*16, SOUT);

    -- Test 5 ----------------------------------------------------
    --   5-bit data, stick even parity, 1 stop
    sout_chk(5, "01010101", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10101010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "01011010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10100101", '0', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 6 ----------------------------------------------------
    --   5-bit data, stick even parity, 1.5 stop
    sout_chk(5, "01010101", '0', 1.5, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10101010", '0', 1.5, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "01011010", '0', 1.5, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10100101", '0', 1.5, True, CLK_PERIOD*16, SOUT);

    -- Test 7 ----------------------------------------------------
    --   5-bit data, stick odd parity, 1 stop
    sout_chk(5, "01010101", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10101010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "01011010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10100101", '1', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 8 ----------------------------------------------------
    --   5-bit data, stick odd parity, 1.5 stop
    sout_chk(5, "01010101", '1', 1.5, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10101010", '1', 1.5, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "01011010", '1', 1.5, True, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10100101", '1', 1.5, True, CLK_PERIOD*16, SOUT);

    -- Test 9 ----------------------------------------------------
    --   5-bit data, no parity, 1 stop
    sout_chk(5, "01010101", '0', 1.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10101010", '0', 1.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(5, "01011010", '0', 1.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10100101", '0', 1.0, false, CLK_PERIOD*16, SOUT);

    -- Test 10 ---------------------------------------------------
    --   5-bit data, no parity, 1.5 stop
    sout_chk(5, "01010101", '0', 1.5, false, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10101010", '0', 1.5, false, CLK_PERIOD*16, SOUT);
    sout_chk(5, "01011010", '0', 1.5, false, CLK_PERIOD*16, SOUT);
    sout_chk(5, "10100101", '0', 1.5, false, CLK_PERIOD*16, SOUT);

    -- Test 11 ---------------------------------------------------
    --   6-bit data, even parity, 1 stop
    sout_chk(6, "01010101", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10101010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "01011010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10100101", '1', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 12 ---------------------------------------------------
    --   6-bit data, even parity, 2 stop
    sout_chk(6, "01010101", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10101010", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "01011010", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10100101", '1', 2.0, True, CLK_PERIOD*16, SOUT);

    -- Test 13 ---------------------------------------------------
    --   6-bit data, odd parity, 1 stop
    sout_chk(6, "01010101", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10101010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "01011010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10100101", '0', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 14 ---------------------------------------------------
    --   6-bit data, odd parity, 2 stop
    sout_chk(6, "01010101", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10101010", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "01011010", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10100101", '0', 2.0, True, CLK_PERIOD*16, SOUT);

    -- Test 15 ---------------------------------------------------
    --   6-bit data, stick even parity, 1 stop
    sout_chk(6, "01010101", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10101010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "01011010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10100101", '0', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 16 ---------------------------------------------------
    --   6-bit data, stick even parity, 2 stop
    sout_chk(6, "01010101", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10101010", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "01011010", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10100101", '0', 2.0, True, CLK_PERIOD*16, SOUT);

    -- Test 17 ---------------------------------------------------
    --   6-bit data, stick odd parity, 1 stop
    sout_chk(6, "01010101", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10101010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "01011010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10100101", '1', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 18 ---------------------------------------------------
    --   6-bit data, stick odd parity, 2 stop
    sout_chk(6, "01010101", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10101010", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "01011010", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10100101", '1', 2.0, True, CLK_PERIOD*16, SOUT);

    -- Test 19 ---------------------------------------------------
    --   6-bit data, no parity, 1 stop
    sout_chk(6, "01010101", '0', 1.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10101010", '0', 1.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(6, "01011010", '0', 1.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10100101", '0', 1.0, false, CLK_PERIOD*16, SOUT);

    -- Test 20 ---------------------------------------------------
    --   6-bit data, no parity, 2 stop
    sout_chk(6, "01010101", '0', 2.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10101010", '0', 2.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(6, "01011010", '0', 2.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(6, "10100101", '0', 2.0, false, CLK_PERIOD*16, SOUT);

    -- Test 21 ---------------------------------------------------
    --   7-bit data, even parity, 1 stop
    sout_chk(7, "01010101", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10101010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "01011010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10100101", '1', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 22 ---------------------------------------------------
    --   7-bit data, even parity, 2 stop
    sout_chk(7, "01010101", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10101010", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "01011010", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10100101", '1', 2.0, True, CLK_PERIOD*16, SOUT);

    -- Test 23 ---------------------------------------------------
    --   7-bit data, odd parity, 1 stop
    sout_chk(7, "01010101", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10101010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "01011010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10100101", '0', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 24 ---------------------------------------------------
    --   7-bit data, odd parity, 2 stop
    sout_chk(7, "01010101", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10101010", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "01011010", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10100101", '0', 2.0, True, CLK_PERIOD*16, SOUT);

    -- Test 25 ---------------------------------------------------
    --   7-bit data, stick even parity, 1 stop
    sout_chk(7, "01010101", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10101010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "01011010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10100101", '0', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 26 ---------------------------------------------------
    --   7-bit data, stick even parity, 2 stop
    sout_chk(7, "01010101", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10101010", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "01011010", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10100101", '0', 2.0, True, CLK_PERIOD*16, SOUT);

    -- Test 27 ---------------------------------------------------
    --   7-bit data, stick odd parity, 1 stop
    sout_chk(7, "01010101", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10101010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "01011010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10100101", '1', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 28 ---------------------------------------------------
    --   7-bit data, stick odd parity, 2 stop
    sout_chk(7, "01010101", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10101010", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "01011010", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10100101", '1', 2.0, True, CLK_PERIOD*16, SOUT);

    -- Test 29 ---------------------------------------------------
    --   7-bit data, no parity, 1 stop
    sout_chk(7, "01010101", '0', 1.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10101010", '0', 1.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(7, "01011010", '0', 1.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10100101", '0', 1.0, false, CLK_PERIOD*16, SOUT);

    -- Test 30 ---------------------------------------------------
    --   7-bit data, no parity, 2 stop
    sout_chk(7, "01010101", '0', 2.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10101010", '0', 2.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(7, "01011010", '0', 2.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(7, "10100101", '0', 2.0, false, CLK_PERIOD*16, SOUT);

    -- Test 31 ---------------------------------------------------
    --   8-bit data, even parity, 1 stop
    sout_chk(8, "01010101", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10101010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "01011010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10100101", '0', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 32 ---------------------------------------------------
    --   8-bit data, even parity, 2 stop
    sout_chk(8, "01010101", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10101010", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "01011010", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10100101", '0', 2.0, True, CLK_PERIOD*16, SOUT);

    -- Test 33 ---------------------------------------------------
    --   8-bit data, odd parity, 1 stop
    sout_chk(8, "01010101", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10101010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "01011010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10100101", '1', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 34 ---------------------------------------------------
    --   8-bit data, odd parity, 2 stop
    sout_chk(8, "01010101", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10101010", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "01011010", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10100101", '1', 2.0, True, CLK_PERIOD*16, SOUT);

    -- Test 35 ---------------------------------------------------
    --   8-bit data, stick even parity, 1 stop
    sout_chk(8, "01010101", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10101010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "01011010", '0', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10100101", '0', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 36 ---------------------------------------------------
    --   8-bit data, stick even parity, 2 stop
    sout_chk(8, "01010101", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10101010", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "01011010", '0', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10100101", '0', 2.0, True, CLK_PERIOD*16, SOUT);

    -- Test 37 ---------------------------------------------------
    --   8-bit data, stick odd parity, 1 stop
    sout_chk(8, "01010101", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10101010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "01011010", '1', 1.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10100101", '1', 1.0, True, CLK_PERIOD*16, SOUT);

    -- Test 38 ---------------------------------------------------
    --   8-bit data, stick odd parity, 2 stop
    sout_chk(8, "01010101", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10101010", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "01011010", '1', 2.0, True, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10100101", '1', 2.0, True, CLK_PERIOD*16, SOUT);

    -- Test 39 ---------------------------------------------------
    --   8-bit data, no parity, 1 stop
    sout_chk(8, "01010101", '0', 1.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10101010", '0', 1.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(8, "01011010", '0', 1.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10100101", '0', 1.0, false, CLK_PERIOD*16, SOUT);

    -- Test 40 ---------------------------------------------------
    --   8-bit data, no parity, 2 stop
    sout_chk(8, "01010101", '0', 2.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10101010", '0', 2.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(8, "01011010", '0', 2.0, false, CLK_PERIOD*16, SOUT);
    sout_chk(8, "10100101", '0', 2.0, false, CLK_PERIOD*16, SOUT);


    -- end of tests ----------------------------------------------
    wait;
  end process Sout_Chk_Proc;
    
-----------------------------------------------------------------------
-- Test UART Transmitter/Receiver Functions
-----------------------------------------------------------------------
  UART_Stim_Proc : process
    variable i : integer;
  begin

    -- Reset and Intialization
    MR <= '1';

    CS <= '0';
    ADSn <= '1';
    A <= "111";
    DIN <= "11111111";
    SIN <= '1';
    CTSn <= '1';
    DCDn <= '1';
    DSRn <= '1';
    RIn  <= '1';

    wait for (9.5*CLK_PERIOD);

    MR <= '0';

    wait for (0.5*CLK_PERIOD);


    wait until falling_edge(PCLK);
    -- Test 1 ----------------------------------------------------
    --   5-bit data, even parity, 1 stop
    TestID <= 1;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 0, 5 data bit (bit[1-0]="00")
    --   bit 0 : 0, 5 data bit (bit[1-0]="00")
    write_reg (LCR,"00011000",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 2 ----------------------------------------------------
    --   5-bit data, even parity, 1.5 stop
    TestID <= 2;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 1.5 stop bit
    --   bit 1 : 0, 5 data bit (bit[1-0]="00")
    --   bit 0 : 0, 5 data bit (bit[1-0]="00")
    write_reg (LCR,"00011100",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 3 ----------------------------------------------------
    --   5-bit data, odd parity, 1 stop
    TestID <= 3;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 0, 5 data bit (bit[1-0]="00")
    --   bit 0 : 0, 5 data bit (bit[1-0]="00")
    write_reg (LCR,"00001000",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 4 ----------------------------------------------------
    --   5-bit data, odd parity, 1.5 stop
    TestID <= 4;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 1.5 stop bit
    --   bit 1 : 0, 5 data bit (bit[1-0]="00")
    --   bit 0 : 0, 5 data bit (bit[1-0]="00")
    write_reg (LCR,"00001100",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 5 ----------------------------------------------------
    --   5-bit data, stick even parity, 1 stop
    TestID <= 5;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 0, 5 data bit (bit[1-0]="00")
    --   bit 0 : 0, 5 data bit (bit[1-0]="00")
    write_reg (LCR,"00111000",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 6 ----------------------------------------------------
    --   5-bit data, stick even parity, 1.5 stop
    TestID <= 6;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 1.5 stop bit
    --   bit 1 : 0, 5 data bit (bit[1-0]="00")
    --   bit 0 : 0, 5 data bit (bit[1-0]="00")
    write_reg (LCR,"00111100",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 7 ----------------------------------------------------
    --   5-bit data, stick odd parity, 1 stop
    TestID <= 7;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 0, 5 data bit (bit[1-0]="00")
    --   bit 0 : 0, 5 data bit (bit[1-0]="00")
    write_reg (LCR,"00101000",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 8 ----------------------------------------------------
    --   5-bit data, stick odd parity, 1.5 stop
    TestID <= 8;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 1.5 stop bit
    --   bit 1 : 0, 5 data bit (bit[1-0]="00")
    --   bit 0 : 0, 5 data bit (bit[1-0]="00")
    write_reg (LCR,"00101100",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 9 ----------------------------------------------------
    --   5-bit data, no parity, 1 stop
    TestID <= 9;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 0, parity disabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 0, 5 data bit (bit[1-0]="00")
    --   bit 0 : 0, 5 data bit (bit[1-0]="00")
    write_reg (LCR,"00000000",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 10 ---------------------------------------------------
    --   5-bit data, stick odd parity, 1.5 stop
    TestID <= 10;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 0, parity disabled
    --   bit 2 : 1, 1.5 stop bit
    --   bit 1 : 0, 5 data bit (bit[1-0]="00")
    --   bit 0 : 0, 5 data bit (bit[1-0]="00")
    write_reg (LCR,"00000100",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 11 ---------------------------------------------------
    --   6-bit data, even parity, 1 stop
    TestID <= 11;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 0, 6 data bit (bit[1-0]="01")
    --   bit 0 : 1, 6 data bit (bit[1-0]="01")
    write_reg (LCR,"00011001",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 12 ---------------------------------------------------
    --   6-bit data, even parity, 2 stop
    TestID <= 12;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 0, 6 data bit (bit[1-0]="01")
    --   bit 0 : 1, 6 data bit (bit[1-0]="01")
    write_reg (LCR,"00011101",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 13 ---------------------------------------------------
    --   6-bit data, odd parity, 1 stop
    TestID <= 13;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 0, 6 data bit (bit[1-0]="01")
    --   bit 0 : 1, 6 data bit (bit[1-0]="01")
    write_reg (LCR,"00001001",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 14 ---------------------------------------------------
    --   6-bit data, odd parity, 2 stop
    TestID <= 14;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 0, 6 data bit (bit[1-0]="01")
    --   bit 0 : 1, 6 data bit (bit[1-0]="01")
    write_reg (LCR,"00001101",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 15 ---------------------------------------------------
    --   6-bit data, stick even parity, 1 stop
    TestID <= 15;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 0, 6 data bit (bit[1-0]="01")
    --   bit 0 : 1, 6 data bit (bit[1-0]="01")
    write_reg (LCR,"00111001",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 16 ---------------------------------------------------
    --   6-bit data, stick even parity, 2 stop
    TestID <= 16;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 0, 6 data bit (bit[1-0]="01")
    --   bit 0 : 1, 6 data bit (bit[1-0]="01")
    write_reg (LCR,"00111101",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 17 ---------------------------------------------------
    --   6-bit data, stick odd parity, 1 stop
    TestID <= 17;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 0, 6 data bit (bit[1-0]="01")
    --   bit 0 : 1, 6 data bit (bit[1-0]="01")
    write_reg (LCR,"00101001",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 18;---------------------------------------------------
    --   6-bit data, stick odd parity, 2 stop
    TestID <= 18;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 0, 6 data bit (bit[1-0]="01")
    --   bit 0 : 1, 6 data bit (bit[1-0]="01")
    write_reg (LCR,"00101101",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 19 ---------------------------------------------------
    --   6-bit data, no parity, 1 stop
    TestID <= 19;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 0, parity disabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 0, 6 data bit (bit[1-0]="01")
    --   bit 0 : 1, 6 data bit (bit[1-0]="01")
    write_reg (LCR,"00000001",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 20 ---------------------------------------------------
    --   6-bit data, stick odd parity, 2 stop
    TestID <= 20;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 0, parity disabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 0, 6 data bit (bit[1-0]="01")
    --   bit 0 : 1, 6 data bit (bit[1-0]="01")
    write_reg (LCR,"00000101",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 21 ---------------------------------------------------
    --   7-bit data, even parity, 1 stop
    TestID <= 21;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 1, 7 data bit (bit[1-0]="10")
    --   bit 0 : 0, 7 data bit (bit[1-0]="10")
    write_reg (LCR,"00011010",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 22 ---------------------------------------------------
    --   7-bit data, even parity, 2 stop
    TestID <= 22;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 1, 7 data bit (bit[1-0]="10")
    --   bit 0 : 0, 7 data bit (bit[1-0]="10")
    write_reg (LCR,"00011110",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 23 ---------------------------------------------------
    --   7-bit data, odd parity, 1 stop
    TestID <= 23;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 1, 7 data bit (bit[1-0]="10")
    --   bit 0 : 0, 7 data bit (bit[1-0]="10")
    write_reg (LCR,"00001010",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 24 ---------------------------------------------------
    --   7-bit data, odd parity, 2 stop
    TestID <= 24;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 1, 7 data bit (bit[1-0]="10")
    --   bit 0 : 0, 7 data bit (bit[1-0]="10")
    write_reg (LCR,"00001110",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 25 ---------------------------------------------------
    --   7-bit data, stick even parity, 1 stop
    TestID <= 25;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 1, 7 data bit (bit[1-0]="10")
    --   bit 0 : 0, 7 data bit (bit[1-0]="10")
    write_reg (LCR,"00111010",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 26 ---------------------------------------------------
    --   7-bit data, stick even parity, 2 stop
    TestID <= 26;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 1, 7 data bit (bit[1-0]="10")
    --   bit 0 : 0, 7 data bit (bit[1-0]="10")
    write_reg (LCR,"00111110",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 27 ---------------------------------------------------
    --   7-bit data, stick odd parity, 1 stop
    TestID <= 27;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 1, 7 data bit (bit[1-0]="10")
    --   bit 0 : 0, 7 data bit (bit[1-0]="10")
    write_reg (LCR,"00101010",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 28;---------------------------------------------------
    --   7-bit data, stick odd parity, 2 stop
    TestID <= 28;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 1, 7 data bit (bit[1-0]="10")
    --   bit 0 : 0, 7 data bit (bit[1-0]="10")
    write_reg (LCR,"00101110",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 29 ---------------------------------------------------
    --   7-bit data, no parity, 1 stop
    TestID <= 29;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 0, parity disabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 1, 7 data bit (bit[1-0]="10")
    --   bit 0 : 0, 7 data bit (bit[1-0]="10")
    write_reg (LCR,"00000010",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 30 ---------------------------------------------------
    --   7-bit data, stick odd parity, 2 stop
    TestID <= 30;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 0, parity disabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 1, 7 data bit (bit[1-0]="10")
    --   bit 0 : 0, 7 data bit (bit[1-0]="10")
    write_reg (LCR,"00000110",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 31 ---------------------------------------------------
    --   8-bit data, even parity, 1 stop
    TestID <= 31;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 1, 8 data bit (bit[1-0]="11")
    --   bit 0 : 1, 8 data bit (bit[1-0]="11")
    write_reg (LCR,"00011011",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 32 ---------------------------------------------------
    --   8-bit data, even parity, 2 stop
    TestID <= 32;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 1, 8 data bit (bit[1-0]="11")
    --   bit 0 : 1, 8 data bit (bit[1-0]="11")
    write_reg (LCR,"00011111",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 33 ---------------------------------------------------
    --   8-bit data, odd parity, 1 stop
    TestID <= 33;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 1, 8 data bit (bit[1-0]="11")
    --   bit 0 : 1, 8 data bit (bit[1-0]="11")
    write_reg (LCR,"00001011",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);


    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 34 ---------------------------------------------------
    --   8-bit data, odd parity, 2 stop
    TestID <= 34;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 1, 8 data bit (bit[1-0]="11")
    --   bit 0 : 1, 8 data bit (bit[1-0]="11")
    write_reg (LCR,"00001111",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 35 ---------------------------------------------------
    --   8-bit data, stick even parity, 1 stop
    TestID <= 35;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 1, 8 data bit (bit[1-0]="11")
    --   bit 0 : 1, 8 data bit (bit[1-0]="11")
    write_reg (LCR,"00111011",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 36 ---------------------------------------------------
    --   8-bit data, stick even parity, 2 stop
    TestID <= 36;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 1, even parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 1, 8 data bit (bit[1-0]="11")
    --   bit 0 : 1, 8 data bit (bit[1-0]="11")
    write_reg (LCR,"00111111",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 37 ---------------------------------------------------
    --   8-bit data, stick odd parity, 1 stop
    TestID <= 37;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 1, 8 data bit (bit[1-0]="11")
    --   bit 0 : 1, 8 data bit (bit[1-0]="11")
    write_reg (LCR,"00101011",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 38;---------------------------------------------------
    --   8-bit data, stick odd parity, 2 stop
    TestID <= 38;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 1, stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 1, parity enabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 1, 8 data bit (bit[1-0]="11")
    --   bit 0 : 1, 8 data bit (bit[1-0]="11")
    write_reg (LCR,"00101111",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 39 ---------------------------------------------------
    --   8-bit data, no parity, 1 stop
    TestID <= 39;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 0, parity disabled
    --   bit 2 : 0, 1 stop bit
    --   bit 1 : 1, 8 data bit (bit[1-0]="11")
    --   bit 0 : 1, 8 data bit (bit[1-0]="11")
    write_reg (LCR,"00000011",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    wait until falling_edge(PCLK);
    -- Test 40 ---------------------------------------------------
    --   8-bit data, stick odd parity, 2 stop
    TestID <= 40;

    i := 1;
    loop
      if (i < WAIT_TIMEOUT) then
        wait for CLK_PERIOD;
        -- Read LSR (check if "TEMT" flag at bit 6 is set)
        read_reg (LSR,regData_readBack,CS,ADSn,RDn,A,DOUT);
        exit when regData_readBack(6) = '1';
        i := i + 1;
      else
        assert (false) report"Data Transmission Failed"
        severity failure;
      end if;
    end loop;
    wait for (16*CLK_PERIOD);

    -- LCR Intialization
    --   bit 6 : 0, do not set break
    --   bit 5 : 0, not stick parity
    --   bit 4 : 0, odd parity selected
    --   bit 3 : 0, parity disabled
    --   bit 2 : 1, 2 stop bit
    --   bit 1 : 1, 8 data bit (bit[1-0]="11")
    --   bit 0 : 1, 8 data bit (bit[1-0]="11")
    write_reg (LCR,"00000111",CS,ADSn,WRn,A,DIN);

    -- Write 1st data to THR
    write_reg (THR,"01010101",CS,ADSn,WRn,A,DIN);

    -- Write 2nd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10101010",CS,ADSn,WRn,A,DIN);

    -- Write 3rd data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"01011010",CS,ADSn,WRn,A,DIN);

    -- Write 4th data to THR
    if (TxRDYn = '1') then
       wait until TxRDYn = '0';
    end if;
    write_reg (THR,"10100101",CS,ADSn,WRn,A,DIN);


    -- end of tests ----------------------------------------------
    assert (false)
      report"End of UART Transmitter Tests ....."
      severity failure;

   end process UART_Stim_Proc;


-- *** end of test bench ***

end;


